/**
 *	class ConsoleAPI		- consoleAPI.h
 *
 *		date	:	13dec2004
 *		author	:	FYMong
 *		purpose	:	API for console operations
 *	
 *		update	:	07mar2005
 *					consoleAPI11b
 *					+ enum BoxBorder { Box, TickBox, TotBox };
 *					+ setsubScreen(BoxBorder,...);
 */
#ifndef consoleAPI_consoleAPI
#define consoleAPI_consoleAPI
#include "pix.h"
#include "point.h"
#include <string>
#include <iostream>
using namespace std;

class cConsole;
class ScreenList;


class ConsoleAPI
{
	public :
		/**
		 *	default console size
		 */
		enum {HeightMax=50,WidthMax=100};
		/**
		 *	box border
		 */
		enum BoxBorder { SlimBox, Box, ThickBox, TotBox };
		/**
		 *	default constuctor
		 */
		ConsoleAPI(unsigned int height=HeightMax,unsigned int width=WidthMax,const string& title="cs2332");
		/**
		 *	destructor
		 */
		virtual ~ConsoleAPI();
		/**
		 *	set screen title
		 */
		virtual void setTitle(const string& title);
		/**
		 *	create sub-screen
		 *		@param	- upper left point, height & width
		 *		return sub-screen id. (0-unsuccessful)
		 */
		virtual unsigned int newSubScreen(const Point& upLeft,unsigned int height,unsigned int width);
		/**
		 *	set sub-screen : border, background
		 *		@param	- sub-screen id., border, border color, background color
		 *		return false if unsuccessful
		 */
		virtual bool setSubScreen(unsigned int scrid=0,const Pix& border=Pix(),Pix::Color bgcolor=Pix::Black);
		/**
		 *	set sub-screen : border, background
		 *		@param	- sub-screen id., border, border color, background color
		 *		return false if unsuccessful
		 */
		virtual bool setSubScreenBox(unsigned int scrid=0,Pix::BoxBorder box=Pix::Box,const Pix& border=Pix(),Pix::Color bgcolor=Pix::Black);
		/**
		 *	clear (sub-)screen 
		 *		@param	- screen id.(0-default screen)
		 */
		virtual bool clearScreen(unsigned int scrid=0);
		/**
		 *	show cursor
		 */
		virtual void showCursor() const;
		/**
		 *	write a point on screen
		 *		@param	- point,symbol, sub-screen id.(0-default screen)
		 *		retrun false if unsuccessful
		 */
		virtual bool showPix(const Point& pt,const Pix& pix,unsigned int scrid=0);
		/**
		 *	write a string on screen
		 *		@param	-	point, string, color, bgcolor
		 */
		virtual bool Post(const Point& pt,const string& str,unsigned int scrid=0,Pix::Color color=Pix::White,Pix::Color bgcolor=Pix::Black);
		/**
		 *	write a string on screen
		 *		@param	-	point, string, color, bgcolor
		 */
		virtual bool PostCStr(const Point& pt,const string& str,unsigned int scrid=0,Pix::Color color=Pix::White,Pix::Color bgcolor=Pix::Black);
		/**
		 *	detect a keyboard hit
		 *		return true/false 
		 */
		virtual bool keyboardHit();
		/**
		 *	get an input character
		 *		return a character
		 */
		virtual char getKey();
		/**
		 *	time control
		 */
		virtual void delay(long msecs);
	private :
		/**
		 *	console
		 */
		cConsole* consolescr;
		/**
		 *	height
		 */
		unsigned int scrheight;
		/**
		 *	width
		 */
		unsigned int scrwidth;
		/**
		 *	list of sub-screens
		 */
		ScreenList* scrlist;
};


#endif
